@@ -142,6 +142,11 @@ typedef struct
   GList  *usable_xinerama_region;
 } ConstraintInfo;
 
+static gboolean do_screen_and_xinerama_relative_constraints (MetaWindow     *window,
+                                                             GList          *region_spanning_rectangles,
+                                                             ConstraintInfo *info,
+                                                             gboolean        check_only);
+
 static gboolean constrain_modal_dialog       (MetaWindow         *window,
                                               ConstraintInfo     *info,
                                               ConstraintPriority  priority,
@@ -338,6 +343,27 @@ meta_window_constrain (MetaWindow          *window,
     g_free (info.borders);
 }
 
+static gboolean
+validate_fullscreen_monitors (MetaWindow *window)
+{
+  gint i;
+
+  if (window->fullscreen_monitors[0] == -1)
+    return FALSE;
+
+  for (i = 0; i < 4; i++)
+    {
+      gint monitor;
+
+      monitor = window->fullscreen_monitors[i];
+
+      if (monitor < 0 || monitor > window->screen->n_xinerama_infos)
+        return FALSE;
+    }
+
+  return TRUE;
+}
+
 static void
 setup_constraint_info (ConstraintInfo      *info,
                        MetaWindow          *window,
@@ -413,7 +439,7 @@ setup_constraint_info (ConstraintInfo      *info,
                                           xinerama_info->number,
                                           &info->work_area_xinerama);
 
-  if (!window->fullscreen || window->fullscreen_monitors[0] == -1)
+  if (!window->fullscreen || !validate_fullscreen_monitors (window))
     {
       info->entire_xinerama = xinerama_info->rect;
     }
@@ -453,13 +479,8 @@ setup_constraint_info (ConstraintInfo      *info,
       window->has_fullscreen_func &&
       !window->fullscreen)
     {
-      /*
-      meta_topic (META_DEBUG_GEOMETRY,
-      */
-      meta_warning (
-                  "Treating resize request of legacy application %s as a "
-                  "fullscreen request\n",
-                  window->desc);
+      g_warning ("Treating resize request of legacy application %s as a "
+                 "fullscreen request", window->desc);
       meta_window_make_fullscreen_internal (window);
     }
 
@@ -549,11 +570,10 @@ place_window_if_needed(MetaWindow     *window,
   if (window->placed || did_placement)
     {
       if (window->maximize_horizontally_after_placement ||
-          window->maximize_vertically_after_placement   ||
-          window->fullscreen_after_placement)
+          window->maximize_vertically_after_placement)
         {
-          /* define a sane saved_rect so that the user can unmaximize or
-           * make unfullscreen to something reasonable.
+          /* define a sane saved_rect so that the user can unmaximize to
+           * something reasonable.
            */
           if (info->current.width >= info->work_area_xinerama.width)
             {
@@ -580,13 +600,6 @@ place_window_if_needed(MetaWindow     *window,
           if (window->frame && !window->fullscreen)
             meta_frame_calc_borders (window->frame, info->borders);
 
-          if (window->fullscreen_after_placement)
-            {
-              window->saved_rect = info->current;
-              window->fullscreen = TRUE;
-              window->fullscreen_after_placement = FALSE;
-            }
-
           window->maximize_horizontally_after_placement = FALSE;
           window->maximize_vertically_after_placement = FALSE;
         }
@@ -796,29 +809,28 @@ constrain_modal_dialog (MetaWindow         *window,
 {
   int x, y;
   MetaWindow *parent = meta_window_get_transient_for (window);
+  MetaRectangle child_rect, parent_rect;
   gboolean constraint_already_satisfied;
 
-  if (!meta_prefs_get_attach_modal_dialogs ())
-    return TRUE;
-  if (window->type != META_WINDOW_MODAL_DIALOG || !parent || parent == window)
+  if (!meta_window_is_attached_dialog (window))
     return TRUE;
 
-  x = parent->rect.x + (parent->rect.width / 2 - info->current.width / 2);
-  y = 0;
-  if (parent->frame)
-    {
-      MetaFrameBorders borders;
+  /* We want to center the dialog on the parent, including the decorations
+     for both of them. info->current is in client X window coordinates, so we need
+     to convert them to frame coordinates, apply the centering and then
+     convert back to client.
+  */
 
-      x += parent->frame->rect.x;
-      y += parent->frame->rect.y;
+  child_rect = info->current;
+  extend_by_frame (window, &child_rect, info->borders);
 
-      meta_frame_calc_borders (parent->frame, &borders);
-      y += borders.total.top;
+  meta_window_get_outer_rect (parent, &parent_rect);
 
-      y += info->borders->visible.top;
-   }
-  else
-    y = parent->rect.y + info->borders->visible.top;
+  child_rect.x = parent_rect.x + (parent_rect.width / 2 - child_rect.width / 2);
+  child_rect.y = parent_rect.y + (parent_rect.height / 2 - child_rect.height / 2);
+  unextend_by_frame (window, &child_rect, info->borders);
+  x = child_rect.x;
+  y = child_rect.y;
 
   constraint_already_satisfied = (x == info->current.x) && (y == info->current.y);
 
@@ -827,7 +839,13 @@ constrain_modal_dialog (MetaWindow         *window,
 
   info->current.y = y;
   info->current.x = x;
-  return TRUE;
+
+  /* The calculated position above may need adjustment to make sure the
+   * dialog does not end up partially off-screen */
+  return do_screen_and_xinerama_relative_constraints (window,
+                                                      info->usable_screen_region,
+                                                      info,
+                                                      check_only);
 }
 
 static gboolean
@@ -1274,8 +1292,7 @@ do_screen_and_xinerama_relative_constraints (
   gboolean exit_early = FALSE, constraint_satisfied;
   MetaRectangle how_far_it_can_be_smushed, min_size, max_size;
 
-#ifdef WITH_VERBOSE_MODE
-  if (meta_is_verbose ())
+  if (meta_check_debug_flags (META_DEBUG_GEOMETRY))
     {
       /* First, log some debugging information */
       char spanning_region[1 + 28 * g_list_length (region_spanning_rectangles)];
@@ -1285,7 +1302,6 @@ do_screen_and_xinerama_relative_constraints (
              meta_rectangle_region_to_string (region_spanning_rectangles, ", ",
                                               spanning_region));
     }
-#endif
 
   /* Determine whether constraint applies; exit if it doesn't */
   how_far_it_can_be_smushed = info->current;
