@@ -203,8 +203,7 @@ meta_window_kill (MetaWindow *window)
         }
       else
         {
-          meta_warning (_("Failed to get hostname: %s\n"),
-                        strerror (errno));
+          g_warning ("Failed to get hostname: %s", strerror (errno));
         }
     }
 
@@ -242,7 +241,7 @@ meta_window_present_delete_dialog (MetaWindow *window, guint32 timestamp)
        * metacity-dialog
        */
 
-      windows = meta_display_list_windows (window->display);
+      windows = meta_display_list_windows (window->display, META_LIST_DEFAULT);
       tmp = windows;
       while (tmp != NULL)
         {
