@@ -37,7 +37,7 @@
 
 #define WINDOW_HAS_TRANSIENT_TYPE(w)                    \
           (w->type == META_WINDOW_DIALOG ||             \
-	   w->type == META_WINDOW_MODAL_DIALOG ||       \
+           w->type == META_WINDOW_MODAL_DIALOG ||       \
            w->type == META_WINDOW_TOOLBAR ||            \
            w->type == META_WINDOW_MENU ||               \
            w->type == META_WINDOW_UTILITY)
@@ -49,7 +49,7 @@
 
 #define WINDOW_IN_STACK(w) (w->stack_position >= 0)
 
-static void stack_sync_to_server (MetaStack *stack);
+static void stack_sync_to_xserver (MetaStack *stack);
 static void meta_window_set_stack_position_no_sync (MetaWindow *window,
                                                     int         position);
 static void stack_do_window_deletions (MetaStack *stack);
@@ -68,14 +68,13 @@ meta_stack_new (MetaScreen *screen)
   stack = g_new (MetaStack, 1);
 
   stack->screen = screen;
-  stack->windows = g_array_new (FALSE, FALSE, sizeof (Window));
+  stack->xwindows = g_array_new (FALSE, FALSE, sizeof (Window));
 
   stack->sorted = NULL;
   stack->added = NULL;
   stack->removed = NULL;
 
   stack->freeze_count = 0;
-  stack->last_root_children_stacked = NULL;
 
   stack->n_positions = 0;
 
@@ -89,15 +88,12 @@ meta_stack_new (MetaScreen *screen)
 void
 meta_stack_free (MetaStack *stack)
 {
-  g_array_free (stack->windows, TRUE);
+  g_array_free (stack->xwindows, TRUE);
 
   g_list_free (stack->sorted);
   g_list_free (stack->added);
   g_list_free (stack->removed);
 
-  if (stack->last_root_children_stacked)
-    g_array_free (stack->last_root_children_stacked, TRUE);
-
   g_free (stack);
 }
 
@@ -105,10 +101,12 @@ void
 meta_stack_add (MetaStack  *stack,
                 MetaWindow *window)
 {
+  g_return_if_fail (!window->override_redirect);
+
   meta_topic (META_DEBUG_STACK, "Adding window %s to the stack\n", window->desc);
 
   if (window->stack_position >= 0)
-    meta_bug ("Window %s had stack position already\n", window->desc);
+    g_error ("Window %s had stack position already", window->desc);
 
   stack->added = g_list_prepend (stack->added, window);
 
@@ -118,7 +116,7 @@ meta_stack_add (MetaStack  *stack,
               "Window %s has stack_position initialized to %d\n",
               window->desc, window->stack_position);
 
-  stack_sync_to_server (stack);
+  stack_sync_to_xserver (stack);
 }
 
 void
@@ -128,8 +126,8 @@ meta_stack_remove (MetaStack  *stack,
   meta_topic (META_DEBUG_STACK, "Removing window %s from the stack\n", window->desc);
 
   if (window->stack_position < 0)
-    meta_bug ("Window %s removed from stack but had no stack position\n",
-              window->desc);
+    g_error ("Window %s removed from stack but had no stack position",
+             window->desc);
 
   /* Set window to top position, so removing it will not leave gaps
    * in the set of positions
@@ -153,7 +151,7 @@ meta_stack_remove (MetaStack  *stack,
     stack->removed = g_list_prepend (stack->removed,
                                      GUINT_TO_POINTER (window->frame->xwindow));
 
-  stack_sync_to_server (stack);
+  stack_sync_to_xserver (stack);
 }
 
 void
@@ -162,7 +160,7 @@ meta_stack_update_layer (MetaStack  *stack,
 {
   stack->need_relayer = TRUE;
 
-  stack_sync_to_server (stack);
+  stack_sync_to_xserver (stack);
 }
 
 void
@@ -171,7 +169,7 @@ meta_stack_update_transient (MetaStack  *stack,
 {
   stack->need_constrain = TRUE;
 
-  stack_sync_to_server (stack);
+  stack_sync_to_xserver (stack);
 }
 
 /* raise/lower within a layer */
@@ -179,19 +177,54 @@ void
 meta_stack_raise (MetaStack  *stack,
                   MetaWindow *window)
 {
-  meta_window_set_stack_position_no_sync (window,
-                                          stack->n_positions - 1);
+  GList *l;
+  int max_stack_position = window->stack_position;
+  MetaWorkspace *workspace;
 
-  stack_sync_to_server (stack);
+  stack_ensure_sorted (stack);
+
+  workspace = meta_window_get_workspace (window);
+  for (l = stack->sorted; l; l = l->next)
+    {
+      MetaWindow *w = (MetaWindow *) l->data;
+      if (meta_window_located_on_workspace (w, workspace) &&
+          w->stack_position > max_stack_position)
+        max_stack_position = w->stack_position;
+    }
+
+  if (max_stack_position == window->stack_position)
+    return;
+
+  meta_window_set_stack_position_no_sync (window, max_stack_position);
+
+  stack_sync_to_xserver (stack);
 }
 
 void
 meta_stack_lower (MetaStack  *stack,
                   MetaWindow *window)
 {
-  meta_window_set_stack_position_no_sync (window, 0);
+  GList *l;
+  int min_stack_position = window->stack_position;
+  MetaWorkspace *workspace;
 
-  stack_sync_to_server (stack);
+  stack_ensure_sorted (stack);
+
+  workspace = meta_window_get_workspace (window);
+  for (l = stack->sorted; l; l = l->next)
+    {
+      MetaWindow *w = (MetaWindow *) l->data;
+      if (meta_window_located_on_workspace (w, workspace) &&
+          w->stack_position < min_stack_position)
+        min_stack_position = w->stack_position;
+    }
+
+  if (min_stack_position == window->stack_position)
+    return;
+
+  meta_window_set_stack_position_no_sync (window, min_stack_position);
+
+  stack_sync_to_xserver (stack);
 }
 
 void
@@ -206,14 +239,14 @@ meta_stack_thaw (MetaStack *stack)
   g_return_if_fail (stack->freeze_count > 0);
 
   stack->freeze_count -= 1;
-  stack_sync_to_server (stack);
+  stack_sync_to_xserver (stack);
 }
 
 static gboolean
 is_focused_foreach (MetaWindow *window,
                     void       *data)
 {
-  if (window == window->display->expected_focus_window)
+  if (window->has_focus)
     {
       *((gboolean*) data) = TRUE;
       return FALSE;
@@ -251,6 +284,16 @@ get_standalone_layer (MetaWindow *window)
       else
         layer = META_LAYER_DOCK;
     }
+  else if (window->type == META_WINDOW_DROPDOWN_MENU ||
+           window->type == META_WINDOW_POPUP_MENU ||
+           window->type == META_WINDOW_TOOLTIP ||
+           window->type == META_WINDOW_NOTIFICATION ||
+           window->type == META_WINDOW_COMBO ||
+           window->type == META_WINDOW_DND ||
+           window->type == META_WINDOW_OVERRIDE_OTHER)
+    {
+      layer = META_LAYER_OVERRIDE_REDIRECT;
+    }
   else
     {
       meta_window_foreach_transient (window,
@@ -261,11 +304,11 @@ get_standalone_layer (MetaWindow *window)
         layer = META_LAYER_BOTTOM;
       else if (window->fullscreen &&
                (focused_transient ||
-                window == window->display->expected_focus_window ||
-                window->display->expected_focus_window == NULL ||
-                (window->display->expected_focus_window != NULL &&
+                window == window->display->focus_window ||
+                window->display->focus_window == NULL ||
+                (window->display->focus_window != NULL &&
                  windows_on_different_xinerama (window,
-                                                window->display->expected_focus_window))))
+                                                window->display->focus_window))))
         layer = META_LAYER_FULLSCREEN;
       else if (window->wm_state_above)
         layer = META_LAYER_TOP;
@@ -302,9 +345,12 @@ get_maximum_layer_in_group (MetaWindow *window)
     {
       MetaWindow *w = tmp->data;
 
-      layer = get_standalone_layer (w);
-      if (layer > max)
-        max = layer;
+      if (!w->override_redirect)
+        {
+          layer = get_standalone_layer (w);
+          if (layer > max)
+            max = layer;
+        }
 
       tmp = tmp->next;
     }
@@ -516,7 +562,8 @@ create_constraints (Constraint **constraints,
               MetaWindow *group_window = tmp2->data;
 
               if (!WINDOW_IN_STACK (group_window) ||
-                  w->screen != group_window->screen)
+                  w->screen != group_window->screen ||
+                  group_window->override_redirect)
                 {
                   tmp2 = tmp2->next;
                   continue;
@@ -642,8 +689,8 @@ ensure_above (MetaWindow *above,
       above->layer < below->layer)
     {
       meta_topic (META_DEBUG_STACK,
-		  "Promoting window %s from layer %u to %u due to contraint\n",
-		  above->desc, above->layer, below->layer);
+                  "Promoting window %s from layer %u to %u due to contraint\n",
+                  above->desc, above->layer, below->layer);
       above->layer = below->layer;
     }
 
@@ -741,7 +788,7 @@ stack_do_window_deletions (MetaStack *stack)
       /* We go from the end figuring removals are more
        * likely to be recent.
        */
-      i = stack->windows->len;
+      i = stack->xwindows->len;
       while (i > 0)
         {
           --i;
@@ -752,9 +799,9 @@ stack_do_window_deletions (MetaStack *stack)
            * both the window->xwindow and window->frame->xwindow
            * in the removal list.
            */
-          if (xwindow == g_array_index (stack->windows, Window, i))
+          if (xwindow == g_array_index (stack->xwindows, Window, i))
             {
-              g_array_remove_index (stack->windows, i);
+              g_array_remove_index (stack->xwindows, i);
               goto next;
             }
         }
@@ -783,10 +830,10 @@ stack_do_window_additions (MetaStack *stack)
                   "Adding %d windows to sorted list\n",
                   n_added);
 
-      old_size = stack->windows->len;
-      g_array_set_size (stack->windows, old_size + n_added);
+      old_size = stack->xwindows->len;
+      g_array_set_size (stack->xwindows, old_size + n_added);
 
-      end = &g_array_index (stack->windows, Window, old_size);
+      end = &g_array_index (stack->xwindows, Window, old_size);
 
       /* stack->added has the most recent additions at the
        * front of the list, so we need to reverse it
@@ -934,115 +981,19 @@ stack_ensure_sorted (MetaStack *stack)
   stack_do_resort (stack);
 }
 
-/**
- * This function is used to avoid raising a window above popup
- * menus and other such things.
- *
- * FIXME This is sort of an expensive function, should probably
- * do something to avoid it. One approach would be to reverse
- * the stacking algorithm to work by placing each window above
- * the others, and start by lowering a window to the bottom
- * (instead of the current way, which works by placing each
- * window below another and starting with a raise)
- */
-static void
-raise_window_relative_to_managed_windows (MetaScreen *screen,
-                                          Window      xwindow)
-{
-
-  Window ignored1, ignored2;
-  Window *children;
-  unsigned int n_children;
-  int i;
-
-  /* Normally XQueryTree() means "must grab server" but here
-   * we don't, since we know we won't manage any new windows
-   * or restack any windows before using the XQueryTree results.
-   */
-
-  meta_error_trap_push (screen->display);
-
-  XQueryTree (screen->display->xdisplay,
-              screen->xroot,
-              &ignored1, &ignored2, &children, &n_children);
-
-  if (meta_error_trap_pop_with_return (screen->display) != Success)
-    {
-      meta_topic (META_DEBUG_STACK,
-                  "Error querying root children to raise window 0x%lx\n",
-                  xwindow);
-      return;
-    }
-
-  /* Children are in order from bottom to top. We want to
-   * find the topmost managed child, then configure
-   * our window to be above it.
-   */
-  i = n_children - 1;
-  while (i >= 0)
-    {
-      if (children[i] == xwindow)
-        {
-          /* Do nothing. This means we're already the topmost managed
-           * window, but it DOES NOT mean we are already just above
-           * the topmost managed window. This is important because if
-           * an override redirect window is up, and we map a new
-           * managed window, the new window is probably above the old
-           * popup by default, and we want to push it below that
-           * popup. So keep looking for a sibling managed window
-           * to be moved below.
-           */
-        }
-      else if (meta_display_lookup_x_window (screen->display,
-                                             children[i]) != NULL)
-        {
-          XWindowChanges changes;
-
-          /* children[i] is the topmost managed child */
-          meta_topic (META_DEBUG_STACK,
-                      "Moving 0x%lx above topmost managed child window 0x%lx\n",
-                      xwindow, children[i]);
-
-          changes.sibling = children[i];
-          changes.stack_mode = Above;
-
-          meta_error_trap_push (screen->display);
-          XConfigureWindow (screen->display->xdisplay,
-                            xwindow,
-                            CWSibling | CWStackMode,
-                            &changes);
-          meta_error_trap_pop (screen->display);
-
-          break;
-        }
-
-      --i;
-    }
-
-  if (i < 0)
-    {
-      /* No sibling to use, just lower ourselves to the bottom
-       * to be sure we're below any override redirect windows.
-       */
-      meta_error_trap_push (screen->display);
-      XLowerWindow (screen->display->xdisplay,
-                    xwindow);
-      meta_error_trap_pop (screen->display);
-    }
-
-  if (children)
-    XFree (children);
-}
-
 /**
  * Order the windows on the X server to be the same as in our structure.
  * We do this using XRestackWindows if we don't know the previous order,
  * or XConfigureWindow on a few particular windows if we do and can figure
  * out the minimum set of changes.  After that, we set __NET_CLIENT_LIST
  * and __NET_CLIENT_LIST_STACKING.
+ *
+ * FIXME: Now that we have a good view of the stacking order on the server
+ * with MetaStackTracker it should be possible to do a simpler and better
+ * job of computing the minimal set of stacking requests needed.
  */
 static void
-stack_sync_to_server (MetaStack *stack)
+stack_sync_to_xserver (MetaStack *stack)
 {
   GArray *stacked;
   GArray *root_children_stacked;
@@ -1056,158 +1007,44 @@ stack_sync_to_server (MetaStack *stack)
 
   stack_ensure_sorted (stack);
 
-  /* Create stacked xwindow arrays.
-   * Painfully, "stacked" is in bottom-to-top order for the
-   * _NET hints, and "root_children_stacked" is in top-to-bottom
-   * order for XRestackWindows()
-   */
+  /* Create stacked xwindow arrays, in bottom-to-top order */
   stacked = g_array_new (FALSE, FALSE, sizeof (Window));
   root_children_stacked = g_array_new (FALSE, FALSE, sizeof (Window));
 
   meta_topic (META_DEBUG_STACK, "Top to bottom: ");
   meta_push_no_msg_prefix ();
 
-  tmp = stack->sorted;
-  while (tmp != NULL)
+  for (tmp = g_list_last (stack->sorted); tmp != NULL; tmp = tmp->prev)
     {
       MetaWindow *w;
 
       w = tmp->data;
 
-      /* remember, stacked is in reverse order (bottom to top) */
+      if (w->unmanaging)
+        continue;
+
+      meta_topic (META_DEBUG_STACK, "%u:%d - %s ",
+                  w->layer, w->stack_position, w->desc);
+
       g_array_prepend_val (stacked, w->xwindow);
 
-      /* build XRestackWindows() array from top to bottom */
       if (w->frame)
         g_array_append_val (root_children_stacked, w->frame->xwindow);
       else
         g_array_append_val (root_children_stacked, w->xwindow);
-
-      meta_topic (META_DEBUG_STACK, "%u:%d - %s ", w->layer, w->stack_position, w->desc);
-
-      tmp = tmp->next;
     }
 
   meta_topic (META_DEBUG_STACK, "\n");
   meta_pop_no_msg_prefix ();
 
-  /* All windows should be in some stacking order */
-  if (stacked->len != stack->windows->len)
-    meta_bug ("%u windows stacked, %u windows exist in stack\n",
-              stacked->len, stack->windows->len);
-
   /* Sync to server */
 
   meta_topic (META_DEBUG_STACK, "Restacking %u windows\n",
               root_children_stacked->len);
 
-  meta_error_trap_push (stack->screen->display);
-
-  if (stack->last_root_children_stacked == NULL)
-    {
-      /* Just impose our stack, we don't know the previous state.
-       * This involves a ton of circulate requests and may flicker.
-       */
-      meta_topic (META_DEBUG_STACK, "Don't know last stack state, restacking everything\n");
-
-      if (root_children_stacked->len > 0)
-        XRestackWindows (stack->screen->display->xdisplay,
-                         (Window *) root_children_stacked->data,
-                         root_children_stacked->len);
-    }
-  else if (root_children_stacked->len > 0)
-    {
-      /* Try to do minimal window moves to get the stack in order */
-      /* A point of note: these arrays include frames not client windows,
-       * so if a client window has changed frame since last_root_children_stacked
-       * was saved, then we may have inefficiency, but I don't think things
-       * break...
-       */
-      const Window *old_stack = (Window *) stack->last_root_children_stacked->data;
-      const Window *new_stack = (Window *) root_children_stacked->data;
-      const int old_len = stack->last_root_children_stacked->len;
-      const int new_len = root_children_stacked->len;
-      const Window *oldp = old_stack;
-      const Window *newp = new_stack;
-      const Window *old_end = old_stack + old_len;
-      const Window *new_end = new_stack + new_len;
-      Window last_window = None;
-
-      while (oldp != old_end &&
-             newp != new_end)
-        {
-          if (*oldp == *newp)
-            {
-              /* Stacks are the same here, move on */
-              ++oldp;
-              last_window = *newp;
-              ++newp;
-            }
-          else if (meta_display_lookup_x_window (stack->screen->display,
-                                                 *oldp) == NULL)
-            {
-              /* *oldp is no longer known to us (probably destroyed),
-               * so we can just skip it
-               */
-              ++oldp;
-            }
-          else
-            {
-              /* Move *newp below last_window */
-              if (last_window == None)
-                {
-                  meta_topic (META_DEBUG_STACK, "Using window 0x%lx as topmost (but leaving it in-place)\n", *newp);
-
-                  raise_window_relative_to_managed_windows (stack->screen,
-                                                            *newp);
-                }
-              else
-                {
-                  /* This means that if last_window is dead, but not
-                   * *newp, then we fail to restack *newp; but on
-                   * unmanaging last_window, we'll fix it up.
-                   */
-
-                  XWindowChanges changes;
-
-                  changes.sibling = last_window;
-                  changes.stack_mode = Below;
-
-                  meta_topic (META_DEBUG_STACK, "Placing window 0x%lx below 0x%lx\n",
-                              *newp, last_window);
-
-                  XConfigureWindow (stack->screen->display->xdisplay,
-                                    *newp,
-                                    CWSibling | CWStackMode,
-                                    &changes);
-                }
-
-              last_window = *newp;
-              ++newp;
-            }
-        }
-
-      if (newp != new_end)
-        {
-          /* Restack remaining windows */
-          meta_topic (META_DEBUG_STACK, "Restacking remaining %d windows\n",
-                        (int) (new_end - newp));
-          /* We need to include an already-stacked window
-           * in the restack call, so we get in the proper position
-           * with respect to it.
-           */
-          if (newp != new_stack)
-            --newp;
-          XRestackWindows (stack->screen->display->xdisplay,
-                           (Window *) newp, new_end - newp);
-        }
-    }
-
-  meta_error_trap_pop (stack->screen->display);
-  /* on error, a window was destroyed; it should eventually
-   * get removed from the stacking list when we unmanage it
-   * and we'll fix stacking at that time.
-   */
+  meta_stack_tracker_restack_managed (stack->screen->stack_tracker,
+                                      (Window *)root_children_stacked->data,
+                                      root_children_stacked->len);
 
   /* Sync _NET_CLIENT_LIST and _NET_CLIENT_LIST_STACKING */
 
@@ -1216,8 +1053,8 @@ stack_sync_to_server (MetaStack *stack)
                    stack->screen->display->atom__NET_CLIENT_LIST,
                    XA_WINDOW,
                    32, PropModeReplace,
-                   (unsigned char *)stack->windows->data,
-                   stack->windows->len);
+                   (unsigned char *)stack->xwindows->data,
+                   stack->xwindows->len);
   XChangeProperty (stack->screen->display->xdisplay,
                    stack->screen->xroot,
                    stack->screen->display->atom__NET_CLIENT_LIST_STACKING,
@@ -1227,12 +1064,7 @@ stack_sync_to_server (MetaStack *stack)
                    stacked->len);
 
   g_array_free (stacked, TRUE);
-
-  if (stack->last_root_children_stacked)
-    g_array_free (stack->last_root_children_stacked, TRUE);
-  stack->last_root_children_stacked = root_children_stacked;
-
-  /* That was scary... */
+  g_array_free (root_children_stacked, TRUE);
 }
 
 MetaWindow*
@@ -1333,90 +1165,45 @@ get_default_focus_window (MetaStack     *stack,
 {
   /* Find the topmost, focusable, mapped, window.
    * not_this_one is being unfocused or going away, so exclude it.
-   * Also, prefer to focus transient parent of not_this_one,
-   * or top window in same group as not_this_one.
    */
 
-  MetaWindow *topmost_dock;
-  MetaWindow *transient_parent;
-  MetaWindow *topmost_in_group;
-  MetaWindow *topmost_overall;
-  MetaGroup *not_this_one_group;
-  GList *link;
-
-  topmost_dock = NULL;
-  transient_parent = NULL;
-  topmost_in_group = NULL;
-  topmost_overall = NULL;
-  if (not_this_one)
-    not_this_one_group = meta_window_get_group (not_this_one);
-  else
-    not_this_one_group = NULL;
+  GList *l;
 
   stack_ensure_sorted (stack);
 
   /* top of this layer is at the front of the list */
-  link = stack->sorted;
-
-  while (link)
+  for (l = stack->sorted; l != NULL; l = l->next)
     {
-      MetaWindow *window = link->data;
+      MetaWindow *window = l->data;
 
-      if (window &&
-          window != not_this_one &&
-          (window->unmaps_pending == 0) &&
-          !window->minimized &&
-          (window->input || window->take_focus) &&
-          (workspace == NULL ||
-           meta_window_located_on_workspace (window, workspace)))
-        {
-          if (topmost_dock == NULL &&
-              window->type == META_WINDOW_DOCK)
-            topmost_dock = window;
+      if (!window)
+        continue;
 
-          if (not_this_one != NULL)
-            {
-              if (transient_parent == NULL &&
-                  not_this_one->xtransient_for != None &&
-                  not_this_one->xtransient_for == window->xwindow &&
-                  (!must_be_at_point ||
-                   window_contains_point (window, root_x, root_y)))
-                transient_parent = window;
+      if (window == not_this_one)
+        continue;
 
-              if (topmost_in_group == NULL &&
-                  not_this_one_group != NULL &&
-                  not_this_one_group == meta_window_get_group (window) &&
-                  (!must_be_at_point ||
-                   window_contains_point (window, root_x, root_y)))
-                topmost_in_group = window;
-            }
+      if (window->unmaps_pending > 0)
+        continue;
 
-          /* Note that DESKTOP windows can be topmost_overall so
-           * we prefer focusing desktop or other windows over
-           * focusing dock, even though docks are stacked higher.
-           */
-          if (topmost_overall == NULL &&
-              window->type != META_WINDOW_DOCK &&
-              (!must_be_at_point ||
-               window_contains_point (window, root_x, root_y)))
-            topmost_overall = window;
+      if (window->unmanaging)
+        continue;
 
-          /* We could try to bail out early here for efficiency in
-           * some cases, but it's just not worth the code.
-           */
-        }
+      if (!(window->input || window->take_focus))
+        continue;
 
-      link = link->next;
+      if (!meta_window_should_be_showing (window))
+        continue;
+
+      if (must_be_at_point && !window_contains_point (window, root_x, root_y))
+        continue;
+
+      if (window->type == META_WINDOW_DOCK)
+        continue;
+
+      return window;
     }
 
-  if (transient_parent)
-    return transient_parent;
-  else if (topmost_in_group)
-    return topmost_in_group;
-  else if (topmost_overall)
-    return topmost_overall;
-  else
-    return topmost_dock;
+  return NULL;
 }
 
 MetaWindow*
@@ -1568,8 +1355,8 @@ meta_stack_set_positions (MetaStack *stack,
 
   if (!lists_contain_same_windows (windows, stack->sorted))
     {
-      meta_warning ("This list of windows has somehow changed; not resetting "
-                    "positions of the windows.\n");
+      g_warning ("This list of windows has somehow changed; not resetting "
+                 "positions of the windows.");
       return;
     }
 
@@ -1591,7 +1378,7 @@ meta_stack_set_positions (MetaStack *stack,
   meta_topic (META_DEBUG_STACK,
               "Reset the stack positions of (nearly) all windows\n");
 
-  stack_sync_to_server (stack);
+  stack_sync_to_xserver (stack);
 }
 
 void
@@ -1653,5 +1440,5 @@ meta_window_set_stack_position (MetaWindow *window,
                                 int         position)
 {
   meta_window_set_stack_position_no_sync (window, position);
-  stack_sync_to_server (window->screen->stack);
+  stack_sync_to_xserver (window->screen->stack);
 }
