@@ -47,14 +47,15 @@
  */
 typedef enum
 {
-  META_LAYER_DESKTOP        = 0,
-  META_LAYER_BOTTOM         = 1,
-  META_LAYER_NORMAL         = 2,
-  META_LAYER_TOP            = 4, /* Same as DOCK; see EWMH and bug 330717 */
-  META_LAYER_DOCK           = 4,
-  META_LAYER_FULLSCREEN     = 5,
-  META_LAYER_FOCUSED_WINDOW = 6,
-  META_LAYER_LAST           = 7
+  META_LAYER_DESKTOP           = 0,
+  META_LAYER_BOTTOM            = 1,
+  META_LAYER_NORMAL            = 2,
+  META_LAYER_TOP               = 4, /* Same as DOCK; see EWMH and bug 330717 */
+  META_LAYER_DOCK              = 4,
+  META_LAYER_FULLSCREEN        = 5,
+  META_LAYER_FOCUSED_WINDOW    = 6,
+  META_LAYER_OVERRIDE_REDIRECT = 7,
+  META_LAYER_LAST              = 8
 } MetaStackLayer;
 
 /**
@@ -73,7 +74,7 @@ struct _MetaStack
    * A sequence of all the Windows (X handles, not MetaWindows) of the windows
    * we manage, sorted in order.  Suitable to be passed into _NET_CLIENT_LIST.
    */
-  GArray *windows;
+  GArray *xwindows;
 
   /** The MetaWindows of the windows we manage, sorted in order. */
   GList *sorted;
@@ -108,12 +109,6 @@ struct _MetaStack
    */
   int freeze_count;
 
-  /**
-   * The last-known stack of all windows, bottom to top.  We cache it here
-   * so that subsequent times we'll be able to do incremental moves.
-   */
-  GArray *last_root_children_stacked;
-
   /**
    * Number of stack positions; same as the length of added, but
    * kept for quick reference.
