@@ -68,8 +68,9 @@ typedef void (* ReloadValueFunc) (MetaWindow    *window,
 typedef enum {
   NONE       = 0,
   LOAD_INIT  = (1 << 0),
-  INIT_ONLY  = (1 << 1),
-  FORCE_INIT = (1 << 2),
+  INCLUDE_OR = (1 << 1),
+  INIT_ONLY  = (1 << 2),
+  FORCE_INIT = (1 << 3),
 } MetaPropHookFlags;
 
 struct _MetaWindowPropHooks {
@@ -102,6 +103,9 @@ meta_window_reload_property_from_xwindow (MetaWindow *window,
   if (!hooks)
     return;
 
+  if ((hooks->flags & INIT_ONLY) && !initial)
+    return;
+
   init_prop_value (window, hooks, &value);
 
   meta_prop_get_values (window->display, xwindow,
@@ -113,7 +117,7 @@ meta_window_reload_property_from_xwindow (MetaWindow *window,
   meta_prop_free_values (&value, 1);
 }
 
-void
+static void
 meta_window_reload_property (MetaWindow *window,
                              Atom        property,
                              gboolean    initial)
@@ -175,7 +179,8 @@ init_prop_value (MetaWindow          *window,
                  MetaWindowPropHooks *hooks,
                  MetaPropValue       *value)
 {
-  if (!hooks || hooks->type == META_PROP_VALUE_INVALID)
+  if (!hooks || hooks->type == META_PROP_VALUE_INVALID ||
+      (window->override_redirect && !(hooks->flags & INCLUDE_OR)))
     {
       value->type = META_PROP_VALUE_INVALID;
       value->atom = None;
@@ -193,7 +198,7 @@ reload_prop_value (MetaWindow          *window,
                    MetaPropValue       *value,
                    gboolean             initial)
 {
-  if (hooks && hooks->reload_func != NULL)
+  if (!(window->override_redirect && !(hooks->flags & INCLUDE_OR)))
     (* hooks->reload_func) (window, value, initial);
 }
 
@@ -217,8 +222,8 @@ complain_about_broken_client (MetaWindow    *window,
                               MetaPropValue *value,
                               gboolean       initial)
 {
-  meta_warning ("Broken client! Window %s changed client leader window or SM client ID\n",
-                window->desc);
+  g_warning ("Broken client! Window %s changed client leader window "
+             "or SM client ID", window->desc);
 }
 
 static void
@@ -226,59 +231,38 @@ reload_net_wm_window_type (MetaWindow    *window,
                            MetaPropValue *value,
                            gboolean       initial)
 {
-  int n_atoms;
-  Atom *atoms;
-  int i;
-
   window->type_atom = None;
-  n_atoms = 0;
-  atoms = NULL;
 
-  meta_prop_get_atom_list (window->display, window->xwindow,
-                           window->display->atom__NET_WM_WINDOW_TYPE,
-                           &atoms, &n_atoms);
-
-  i = 0;
-  while (i < n_atoms)
+  if (value->type != META_PROP_VALUE_INVALID)
     {
-      /* We break as soon as we find one we recognize,
-       * supposed to prefer those near the front of the list
-       */
-      if (atoms[i] == window->display->atom__NET_WM_WINDOW_TYPE_DESKTOP ||
-          atoms[i] == window->display->atom__NET_WM_WINDOW_TYPE_DOCK ||
-          atoms[i] == window->display->atom__NET_WM_WINDOW_TYPE_TOOLBAR ||
-          atoms[i] == window->display->atom__NET_WM_WINDOW_TYPE_MENU ||
-          atoms[i] == window->display->atom__NET_WM_WINDOW_TYPE_DIALOG ||
-          atoms[i] == window->display->atom__NET_WM_WINDOW_TYPE_NORMAL ||
-          atoms[i] == window->display->atom__NET_WM_WINDOW_TYPE_UTILITY ||
-          atoms[i] == window->display->atom__NET_WM_WINDOW_TYPE_SPLASH)
+      int i;
+
+      for (i = 0; i < value->v.atom_list.n_atoms; i++)
         {
-          window->type_atom = atoms[i];
-          break;
+          Atom atom = value->v.atom_list.atoms[i];
+
+          /* We break as soon as we find one we recognize,
+           * supposed to prefer those near the front of the list
+           */
+          if (atom == window->display->atom__NET_WM_WINDOW_TYPE_DESKTOP ||
+              atom == window->display->atom__NET_WM_WINDOW_TYPE_DOCK ||
+              atom == window->display->atom__NET_WM_WINDOW_TYPE_TOOLBAR ||
+              atom == window->display->atom__NET_WM_WINDOW_TYPE_MENU ||
+              atom == window->display->atom__NET_WM_WINDOW_TYPE_UTILITY ||
+              atom == window->display->atom__NET_WM_WINDOW_TYPE_SPLASH ||
+              atom == window->display->atom__NET_WM_WINDOW_TYPE_DIALOG ||
+              atom == window->display->atom__NET_WM_WINDOW_TYPE_DROPDOWN_MENU ||
+              atom == window->display->atom__NET_WM_WINDOW_TYPE_POPUP_MENU ||
+              atom == window->display->atom__NET_WM_WINDOW_TYPE_TOOLTIP ||
+              atom == window->display->atom__NET_WM_WINDOW_TYPE_NOTIFICATION ||
+              atom == window->display->atom__NET_WM_WINDOW_TYPE_COMBO ||
+              atom == window->display->atom__NET_WM_WINDOW_TYPE_DND ||
+              atom == window->display->atom__NET_WM_WINDOW_TYPE_NORMAL)
+            {
+              window->type_atom = atom;
+              break;
+            }
         }
-
-      ++i;
-    }
-
-  meta_XFree (atoms);
-
-  if (meta_is_verbose ())
-    {
-      char *str;
-
-      str = NULL;
-      if (window->type_atom != None)
-        {
-          meta_error_trap_push (window->display);
-          str = XGetAtomName (window->display->xdisplay, window->type_atom);
-          meta_error_trap_pop (window->display);
-        }
-
-      meta_verbose ("Window %s type atom %s\n", window->desc,
-                    str ? str : "(none)");
-
-      if (str)
-        meta_XFree (str);
     }
 
   meta_window_recalc_window_type (window);
@@ -310,23 +294,14 @@ reload_kwm_win_icon (MetaWindow    *window,
   reload_icon (window, window->display->atom__KWM_WIN_ICON);
 }
 
-static gboolean
-gtk_border_equal (GtkBorder *a,
-                  GtkBorder *b)
-{
-  return (a->left == b->left &&
-          a->right == b->right &&
-          a->top == b->top &&
-          a->bottom == b->bottom);
-}
-
 static void
 meta_window_set_custom_frame_extents (MetaWindow *window,
                                       GtkBorder  *extents)
 {
   if (extents)
     {
-      if (window->has_custom_frame_extents && gtk_border_equal (&window->custom_frame_extents, extents))
+      if (window->has_custom_frame_extents &&
+          memcmp (&window->custom_frame_extents, extents, sizeof (GtkBorder)) == 0)
         return;
 
       window->has_custom_frame_extents = TRUE;
@@ -372,6 +347,36 @@ reload_gtk_frame_extents (MetaWindow    *window,
     }
 }
 
+static void
+reload_icon_geometry (MetaWindow    *window,
+                      MetaPropValue *value,
+                      gboolean       initial)
+{
+  if (value->type != META_PROP_VALUE_INVALID)
+    {
+      if (value->v.cardinal_list.n_cardinals != 4)
+        {
+          meta_verbose ("_NET_WM_ICON_GEOMETRY on %s has %d values instead of 4\n",
+                        window->desc, value->v.cardinal_list.n_cardinals);
+        }
+      else
+        {
+          MetaRectangle geometry;
+
+          geometry.x = (int)value->v.cardinal_list.cardinals[0];
+          geometry.y = (int)value->v.cardinal_list.cardinals[1];
+          geometry.width = (int)value->v.cardinal_list.cardinals[2];
+          geometry.height = (int)value->v.cardinal_list.cardinals[3];
+
+          meta_window_set_icon_geometry (window, &geometry);
+        }
+    }
+  else
+    {
+      meta_window_set_icon_geometry (window, NULL);
+    }
+}
+
 static void
 reload_struts (MetaWindow    *window,
                MetaPropValue *value,
@@ -400,8 +405,9 @@ reload_net_wm_pid (MetaWindow    *window,
       gulong cardinal = (int) value->v.cardinal;
 
       if (cardinal <= 0)
-        meta_warning (_("Application set a bogus _NET_WM_PID %lu\n"),
-                      cardinal);
+        {
+          g_warning ("Application set a bogus _NET_WM_PID %lu", cardinal);
+        }
       else
         {
           window->net_wm_pid = cardinal;
@@ -430,6 +436,8 @@ reload_net_wm_user_time_window (MetaWindow    *window,
 {
   if (value->type != META_PROP_VALUE_INVALID)
     {
+      MetaWindow *prev_owner;
+
       /* Unregister old NET_WM_USER_TIME_WINDOW */
       if (window->user_time_window != None)
         {
@@ -442,6 +450,15 @@ reload_net_wm_user_time_window (MetaWindow    *window,
                         NoEventMask);
         }
 
+      /* Ensure the new user time window is not used on another MetaWindow,
+       * and unset its user time window if that is the case.
+       */
+      prev_owner = meta_display_lookup_x_window (window->display, value->v.xwindow);
+      if (prev_owner && prev_owner->user_time_window == value->v.xwindow)
+        {
+          meta_display_unregister_x_window (window->display, value->v.xwindow);
+          prev_owner->user_time_window = None;
+        }
 
       /* Obtain the new NET_WM_USER_TIME_WINDOW and register it */
       window->user_time_window = value->v.xwindow;
@@ -516,11 +533,10 @@ owner_of_process (pid_t process, uid_t *result)
 #define MAX_TITLE_LENGTH 512
 
 /**
- * Called by set_window_title and set_icon_title to set the value of
- * *target to title. It required and atom is set, it will update the
- * appropriate property.
+ * Called by set_window_title() to set the value of @target to @title.
+ * If required and @atom is set, it will update the appropriate property.
  *
- * Returns TRUE if a new title was set.
+ * Returns: %TRUE if a new title was set.
  */
 static gboolean
 set_title_text (MetaWindow  *window,
@@ -705,63 +721,76 @@ reload_wm_name (MetaWindow    *window,
 }
 
 static void
-set_icon_title (MetaWindow *window,
-                const char *title)
+meta_window_set_opaque_region (MetaWindow     *window,
+                               cairo_region_t *region)
 {
-  gboolean modified =
-    set_title_text (window,
-                    window->using_net_wm_visible_icon_name,
-                    title,
-                    window->display->atom__NET_WM_VISIBLE_ICON_NAME,
-                    &window->icon_name);
-  window->using_net_wm_visible_icon_name = modified;
+  if (cairo_region_equal (window->opaque_region, region))
+    return;
+
+  g_clear_pointer (&window->opaque_region, cairo_region_destroy);
+
+  if (region != NULL)
+    window->opaque_region = cairo_region_reference (region);
+
+  meta_compositor_window_shape_changed (window->display->compositor, window);
 }
 
 static void
-reload_net_wm_icon_name (MetaWindow    *window,
-                         MetaPropValue *value,
-                         gboolean       initial)
+reload_opaque_region (MetaWindow    *window,
+                      MetaPropValue *value,
+                      gboolean       initial)
 {
-  if (value->type != META_PROP_VALUE_INVALID)
-    {
-      set_icon_title (window, value->v.str);
-      window->using_net_wm_icon_name = TRUE;
+  int nitems, nrects, rect_index, i;
+  gulong *region;
+  cairo_rectangle_int_t *rects;
+  cairo_region_t *opaque_region;
 
-      meta_verbose ("Using _NET_WM_ICON_NAME for new title of %s: \"%s\"\n",
-                    window->desc, window->title);
-    }
-  else
+  if (value->type == META_PROP_VALUE_INVALID)
     {
-      set_icon_title (window, NULL);
-      window->using_net_wm_icon_name = FALSE;
-      if (!initial)
-        meta_window_reload_property (window, XA_WM_ICON_NAME, FALSE);
-    }
-}
-
-static void
-reload_wm_icon_name (MetaWindow    *window,
-                     MetaPropValue *value,
-                     gboolean       initial)
-{
-  if (window->using_net_wm_icon_name)
-    {
-      meta_verbose ("Ignoring WM_ICON_NAME \"%s\" as _NET_WM_ICON_NAME is set\n",
-                    value->v.str);
+      meta_window_set_opaque_region (window, NULL);
       return;
     }
 
-  if (value->type != META_PROP_VALUE_INVALID)
-    {
-      set_icon_title (window, value->v.str);
+  nitems = value->v.cardinal_list.n_cardinals;
+  region = value->v.cardinal_list.cardinals;
 
-      meta_verbose ("Using WM_ICON_NAME for new title of %s: \"%s\"\n",
-                    window->desc, window->title);
-    }
-  else
+  if (nitems % 4 != 0)
     {
-      set_icon_title (window, NULL);
+      meta_verbose ("_NET_WM_OPAQUE_REGION does not have a list of 4-tuples.");
+      meta_window_set_opaque_region (window, NULL);
+      return;
     }
+
+  /* empty region */
+  if (nitems == 0)
+    {
+      meta_window_set_opaque_region (window, NULL);
+      return;
+    }
+
+  nrects = nitems / 4;
+  rects = g_new (cairo_rectangle_int_t, nrects);
+  rect_index = i = 0;
+
+  while (i < nitems)
+    {
+      cairo_rectangle_int_t *rect;
+
+      rect = &rects[rect_index];
+
+      rect->x = region[i++];
+      rect->y = region[i++];
+      rect->width = region[i++];
+      rect->height = region[i++];
+
+      rect_index++;
+    }
+
+  opaque_region = cairo_region_create_rectangles (rects, nrects);
+  g_free (rects);
+
+  meta_window_set_opaque_region (window, opaque_region);
+  cairo_region_destroy (opaque_region);
 }
 
 static void
@@ -814,7 +843,7 @@ reload_net_wm_state (MetaWindow    *window,
       else if (value->v.atom_list.atoms[i] == window->display->atom__NET_WM_STATE_SKIP_PAGER)
         window->wm_state_skip_pager = TRUE;
       else if (value->v.atom_list.atoms[i] == window->display->atom__NET_WM_STATE_FULLSCREEN)
-        window->fullscreen_after_placement = TRUE;
+        window->fullscreen = TRUE;
       else if (value->v.atom_list.atoms[i] == window->display->atom__NET_WM_STATE_ABOVE)
         window->wm_state_above = TRUE;
       else if (value->v.atom_list.atoms[i] == window->display->atom__NET_WM_STATE_BELOW)
@@ -840,6 +869,7 @@ reload_mwm_hints (MetaWindow    *window,
                   gboolean       initial)
 {
   MotifWmHints *hints;
+  gboolean decorated;
 
   window->mwm_decorated = TRUE;
   window->mwm_border_only = FALSE;
@@ -857,6 +887,7 @@ reload_mwm_hints (MetaWindow    *window,
     }
 
   hints = value->v.motif_hints;
+  decorated = window->decorated;
 
   /* We support those MWM hints deemed non-stupid */
 
@@ -956,6 +987,9 @@ reload_mwm_hints (MetaWindow    *window,
                          META_QUEUE_MOVE_RESIZE |
                          /* because ensure/destroy frame may unmap: */
                          META_QUEUE_CALC_SHOWING);
+
+      if (decorated != window->decorated)
+        g_object_notify (G_OBJECT (window), "decorated");
     }
 }
 
@@ -1049,6 +1083,8 @@ reload_update_counter (MetaWindow    *window,
     {
       XSyncCounter counter = value->v.xcounter;
 
+      meta_window_destroy_sync_request_alarm (window);
+
       window->sync_request_counter = counter;
       meta_verbose ("Window has _NET_WM_SYNC_REQUEST_COUNTER 0x%lx\n",
                     window->sync_request_counter);
@@ -1577,21 +1613,47 @@ reload_transient_for (MetaWindow    *window,
                       MetaPropValue *value,
                       gboolean       initial)
 {
-  window->xtransient_for = None;
+  MetaWindow *parent = NULL;
+  Window transient_for, old_transient_for;
 
   if (value->type != META_PROP_VALUE_INVALID)
-    window->xtransient_for = value->v.xwindow;
-
-  /* Make sure transient_for is valid */
-  if (window->xtransient_for != None &&
-      meta_display_lookup_x_window (window->display,
-                                    window->xtransient_for) == NULL)
     {
-      meta_warning (_("Invalid WM_TRANSIENT_FOR window 0x%lx specified "
-                      "for %s.\n"),
-                    window->xtransient_for, window->desc);
-      window->xtransient_for = None;
+      transient_for = value->v.xwindow;
+
+      parent = meta_display_lookup_x_window (window->display, transient_for);
+      if (!parent)
+        {
+          g_warning ("Invalid WM_TRANSIENT_FOR window 0x%lx specified for %s.",
+                     transient_for, window->desc);
+          transient_for = None;
+        }
+
+      /* Make sure there is not a loop */
+      while (parent)
+        {
+          if (parent == window)
+            {
+              g_warning ("WM_TRANSIENT_FOR window 0x%lx for %s would create loop.",
+                         transient_for, window->desc);
+              transient_for = None;
+              break;
+            }
+
+          parent = meta_display_lookup_x_window (parent->display,
+                                                 parent->xtransient_for);
+        }
     }
+  else
+    transient_for = None;
+
+  if (transient_for == window->xtransient_for)
+    return;
+
+  if (meta_window_appears_focused (window) && window->xtransient_for != None)
+    meta_window_propagate_focus_appearance (window, FALSE);
+
+  old_transient_for = window->xtransient_for;
+  window->xtransient_for = transient_for;
 
   window->transient_parent_is_root_window =
     window->xtransient_for == window->screen->xroot;
@@ -1605,8 +1667,28 @@ reload_transient_for (MetaWindow    *window,
   /* may now be a dialog */
   meta_window_recalc_window_type (window);
 
+  if (!window->constructing)
+    {
+      /* If the window attaches, detaches, or changes attached
+       * parents, we need to destroy the MetaWindow and let a new one
+       * be created (which happens as a side effect of
+       * meta_window_unmanage()). The condition below is correct
+       * because we know window->xtransient_for has changed.
+       */
+      if (window->attached || meta_window_should_attach_to_parent (window))
+        {
+          guint32 timestamp;
+
+          window->xtransient_for = old_transient_for;
+          timestamp = meta_display_get_current_time_roundtrip (window->display);
+          meta_window_unmanage (window, timestamp);
+          return;
+        }
+    }
+
   /* update stacking constraints */
-  meta_stack_update_transient (window->screen->stack, window);
+  if (!window->override_redirect)
+    meta_stack_update_transient (window->screen->stack, window);
 
   /* possibly change its group. We treat being a window's transient as
    * equivalent to making it your group leader, to work around shortcomings
@@ -1617,8 +1699,11 @@ reload_transient_for (MetaWindow    *window,
       window->xtransient_for != window->xgroup_leader)
     meta_window_group_leader_changed (window);
 
-  if (!window->constructing)
-    meta_window_queue (window, META_QUEUE_MOVE_RESIZE);
+  if (!window->constructing && !window->override_redirect)
+    meta_window_queue (window, META_QUEUE_MOVE_RESIZE | META_QUEUE_CALC_SHOWING);
+
+  if (meta_window_appears_focused (window) && window->xtransient_for != None)
+    meta_window_propagate_focus_appearance (window, TRUE);
 }
 
 static void
@@ -1647,6 +1732,27 @@ reload_gtk_theme_variant (MetaWindow    *window,
     }
 }
 
+static void
+reload_window_opacity (MetaWindow    *window,
+                       MetaPropValue *value,
+                       gboolean       initial)
+
+{
+  guint opacity;
+
+  opacity = 0xffffffff;
+
+  if (value->type != META_PROP_VALUE_INVALID)
+    opacity = value->v.cardinal;
+
+  if (window->opacity == opacity)
+    return;
+
+  window->opacity = opacity;
+
+  meta_compositor_window_opacity_changed (window->display->compositor, window);
+}
+
 /**
  * Initialises the property hooks system.  Each row in the table named "hooks"
  * represents an action to take when a property is found on a newly-created
@@ -1664,48 +1770,55 @@ reload_gtk_theme_variant (MetaWindow    *window,
 void
 meta_display_init_window_prop_hooks (MetaDisplay *display)
 {
+  /* The ordering here is significant for the properties we load
+   * initially: they are roughly ordered in the order we want them to
+   * be gotten. We want to get window name and class first so we can
+   * use them in error messages and such. However, name is modified
+   * depending on wm_client_machine, so push it slightly sooner.
+   *
+   * For override-redirect windows, we pay attention to:
+   *
+   *  - properties that identify the window: useful for debugging
+   *    purposes.
+   *  - NET_WM_WINDOW_TYPE: can be used to do appropriate handling
+   *    for different types of override-redirect windows.
+   */
   MetaWindowPropHooks hooks[] = {
     {
       display->atom_WM_CLIENT_MACHINE,
       META_PROP_VALUE_STRING,
       reload_wm_client_machine,
-      LOAD_INIT
+      LOAD_INIT | INCLUDE_OR
     },
     {
       display->atom__NET_WM_NAME,
       META_PROP_VALUE_UTF8,
       reload_net_wm_name,
-      LOAD_INIT
+      LOAD_INIT | INCLUDE_OR
     },
     {
       XA_WM_CLASS,
       META_PROP_VALUE_CLASS_HINT,
       reload_wm_class,
-      LOAD_INIT
+      LOAD_INIT | INCLUDE_OR
     },
     {
       display->atom__NET_WM_PID,
       META_PROP_VALUE_CARDINAL,
       reload_net_wm_pid,
-      LOAD_INIT
+      LOAD_INIT | INCLUDE_OR
     },
     {
       XA_WM_NAME,
       META_PROP_VALUE_TEXT_PROPERTY,
       reload_wm_name,
-      LOAD_INIT
+      LOAD_INIT | INCLUDE_OR
     },
     {
-      display->atom__NET_WM_ICON_NAME,
-      META_PROP_VALUE_UTF8,
-      reload_net_wm_icon_name,
-      LOAD_INIT
-    },
-    {
-      XA_WM_ICON_NAME,
-      META_PROP_VALUE_TEXT_PROPERTY,
-      reload_wm_icon_name,
-      LOAD_INIT
+      display->atom__NET_WM_OPAQUE_REGION,
+      META_PROP_VALUE_CARDINAL_LIST,
+      reload_opaque_region,
+      LOAD_INIT | INCLUDE_OR
     },
     {
       display->atom__NET_WM_DESKTOP,
@@ -1723,7 +1836,7 @@ meta_display_init_window_prop_hooks (MetaDisplay *display)
       display->atom__NET_WM_SYNC_REQUEST_COUNTER,
       META_PROP_VALUE_SYNC_COUNTER,
       reload_update_counter,
-      LOAD_INIT
+      LOAD_INIT | INCLUDE_OR
     },
     {
       XA_WM_NORMAL_HINTS,
@@ -1785,12 +1898,6 @@ meta_display_init_window_prop_hooks (MetaDisplay *display)
       reload_net_wm_user_time_window,
       LOAD_INIT
     },
-    {
-      display->atom_WM_STATE,
-      META_PROP_VALUE_INVALID,
-      NULL,
-      NONE
-    },
     {
       display->atom__NET_WM_ICON,
       META_PROP_VALUE_INVALID,
@@ -1803,6 +1910,12 @@ meta_display_init_window_prop_hooks (MetaDisplay *display)
       reload_kwm_win_icon,
       NONE
     },
+    {
+      display->atom__NET_WM_ICON_GEOMETRY,
+      META_PROP_VALUE_CARDINAL_LIST,
+      reload_icon_geometry,
+      LOAD_INIT
+    },
     {
       display->atom_WM_CLIENT_LEADER,
       META_PROP_VALUE_INVALID,
@@ -1825,7 +1938,7 @@ meta_display_init_window_prop_hooks (MetaDisplay *display)
       display->atom__NET_WM_WINDOW_TYPE,
       META_PROP_VALUE_ATOM_LIST,
       reload_net_wm_window_type,
-      LOAD_INIT | FORCE_INIT
+      LOAD_INIT | INCLUDE_OR | FORCE_INIT
     },
     {
       display->atom__NET_WM_STRUT,
@@ -1839,6 +1952,12 @@ meta_display_init_window_prop_hooks (MetaDisplay *display)
       reload_struts,
       NONE
     },
+    {
+      display->atom__NET_WM_WINDOW_OPACITY,
+      META_PROP_VALUE_CARDINAL,
+      reload_window_opacity,
+      LOAD_INIT | INCLUDE_OR
+    },
     {
       0
     },
