@@ -56,6 +56,9 @@ struct _ThemeViewerWindow
   const gchar      *theme_variant;
   gboolean          composited;
 
+  GtkWidget        *scale_button;
+  gint              scale;
+
   MetaFrameType     frame_type;
   MetaFrameFlags    frame_flags;
 
@@ -301,12 +304,16 @@ get_icon (gint size)
 static void
 get_client_width_and_height (GtkWidget         *widget,
                              ThemeViewerWindow *window,
+                             gint               widget_scale,
                              gint              *width,
                              gint              *height)
 {
   *width = gtk_widget_get_allocated_width (widget) - PADDING * 2;
   *height = gtk_widget_get_allocated_height (widget) - PADDING * 2;
 
+  *width /= 1.0 / widget_scale;
+  *height /= 1.0 / widget_scale;
+
   *width -= window->borders.total.left + window->borders.total.right;
   *height -= window->borders.total.top + window->borders.total.bottom;
 }
@@ -323,6 +330,7 @@ update_button_state (MetaButtonType type,
   GdkDevice *device;
   gint x;
   gint y;
+  gint widget_scale;
 
   window = THEME_VIEWER_WINDOW (user_data);
   state = META_BUTTON_STATE_NORMAL;
@@ -337,6 +345,10 @@ update_button_state (MetaButtonType type,
   x -= PADDING;
   y -= PADDING;
 
+  widget_scale = gtk_widget_get_scale_factor (GTK_WIDGET (window));
+  x /= 1.0 / widget_scale;
+  y /= 1.0 / widget_scale;
+
   if (x >= rect.x && x < (rect.x + rect.width) &&
       y >= rect.y && y < (rect.y + rect.height))
     {
@@ -662,6 +674,7 @@ theme_combo_box_changed_cb (GtkComboBox       *combo_box,
 
   meta_theme_load (window->theme, theme, NULL);
   meta_theme_set_composited (window->theme, window->composited);
+  meta_theme_set_scale (window->theme, window->scale);
 
   update_frame_flags (window);
   update_button_layout (window);
@@ -700,6 +713,7 @@ theme_box_draw_cb (GtkWidget         *widget,
                    cairo_t           *cr,
                    ThemeViewerWindow *window)
 {
+  gint widget_scale;
   gint client_width;
   gint client_height;
 
@@ -710,7 +724,9 @@ theme_box_draw_cb (GtkWidget         *widget,
   cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
   theme_box_draw_grid (widget, cr);
 
-  get_client_width_and_height (widget, window, &client_width, &client_height);
+  widget_scale = gtk_widget_get_scale_factor (widget);
+  get_client_width_and_height (widget, window, widget_scale,
+                               &client_width, &client_height);
 
   if (!window->mini_icon)
     window->mini_icon = get_icon (MINI_ICON_SIZE);
@@ -719,12 +735,18 @@ theme_box_draw_cb (GtkWidget         *widget,
     window->icon = get_icon (ICON_SIZE);
 
   cairo_translate (cr, PADDING, PADDING);
+
+  cairo_save (cr);
+  cairo_scale (cr, 1.0 / widget_scale, 1.0 / widget_scale);
+
   meta_theme_draw_frame (window->theme, window->theme_variant, cr,
                          window->frame_type, window->frame_flags,
                          client_width, client_height, "Metacity Theme Viewer",
                          update_button_state, window,
                          window->mini_icon, window->icon);
 
+  cairo_restore (cr);
+
   return TRUE;
 }
 
@@ -821,6 +843,28 @@ composited_state_set_cb (GtkSwitch         *widget,
 
   meta_theme_set_composited (window->theme, active);
 
+  update_frame_borders (window);
+
+  gtk_widget_queue_draw (window->theme_box);
+}
+
+static void
+scale_changed_cb (GtkSpinButton     *spin_button,
+                  ThemeViewerWindow *window)
+{
+  gint scale;
+
+  scale = (gint) gtk_spin_button_get_value (spin_button);
+
+  if (window->scale == scale)
+    return;
+
+  window->scale = scale;
+
+  meta_theme_set_scale (window->theme, scale);
+
+  update_frame_borders (window);
+
   gtk_widget_queue_draw (window->theme_box);
 }
 
@@ -895,6 +939,9 @@ theme_viewer_window_class_init (ThemeViewerWindowClass *window_class)
   gtk_widget_class_bind_template_callback (widget_class, frame_type_combo_box_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, composited_state_set_cb);
 
+  gtk_widget_class_bind_template_child (widget_class, ThemeViewerWindow, scale_button);
+  gtk_widget_class_bind_template_callback (widget_class, scale_changed_cb);
+
   gtk_widget_class_bind_template_callback (widget_class, notebook_switch_page_cb);
 
   gtk_widget_class_bind_template_child (widget_class, ThemeViewerWindow, benchmark_frame);
@@ -910,12 +957,15 @@ static void
 theme_viewer_window_init (ThemeViewerWindow *window)
 {
   window->composited = TRUE;
+  window->scale = gtk_widget_get_scale_factor (GTK_WIDGET (window));
 
   gtk_widget_init_template (GTK_WIDGET (window));
   gtk_window_set_titlebar (GTK_WINDOW (window), window->header_bar);
 
   gtk_widget_add_events (window->theme_box, GDK_POINTER_MOTION_MASK);
 
+  gtk_spin_button_set_value (GTK_SPIN_BUTTON (window->scale_button), window->scale);
+
   type_combo_box_changed_cb (GTK_COMBO_BOX (window->type_combo_box), window);
 
   gtk_label_set_xalign (GTK_LABEL (window->load_time), 0.0);
